package core_kernel

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Bounded_int_table.With_keySource

Parameters

module Key : sig ... end

Signature

Sourcetype 'data t = (Key.t, 'data) table

Serialization of a bounded int table using bin_io or sexp preserves num_keys, but only takes space proportional to the length of the table.

include Core.Bin_prot.Binable.S1 with type 'data t := 'data t
Sourceval bin_size_t : ('a, 'a t) Bin_prot.Size.sizer1
Sourceval bin_write_t : ('a, 'a t) Bin_prot.Write.writer1
Sourceval bin_read_t : ('a, 'a t) Bin_prot.Read.reader1
Sourceval __bin_read_t__ : ('a, int -> 'a t) Bin_prot.Read.reader1
Sourceval bin_writer_t : ('a, 'a t) Bin_prot.Type_class.S1.writer
Sourceval bin_reader_t : ('a, 'a t) Bin_prot.Type_class.S1.reader
Sourceval bin_t : ('a, 'a t) Bin_prot.Type_class.S1.t
include Sexplib0.Sexpable.S1 with type 'data t := 'data t
Sourceval t_of_sexp : (Sexplib0.Sexp.t -> 'a) -> Sexplib0.Sexp.t -> 'a t
Sourceval sexp_of_t : ('a -> Sexplib0.Sexp.t) -> 'a t -> Sexplib0.Sexp.t
Sourceval create : num_keys:int -> 'data t
Sourceval of_alist : (Key.t * 'data) list -> 'data t Core.Or_error.t

of_alist returns a table whose maximum allowed key is the maximum key in the input list.

Sourceval of_alist_exn : (Key.t * 'data) list -> 'data t