package core_kernel

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Hash_heapSource

A hash-heap is a combination of a heap and a hashtable that supports constant time lookup, and log(n) time removal and replacement of elements in addition to the normal heap operations.

Sourcemodule type S = sig ... end
Sourcemodule Make (Key : Core.Hashtbl.Key_plain) : S with module Key = Key