package core_kernel

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Nonempty_listSource

Sourcetype 'a t =
  1. | :: of 'a * 'a list

A 'a t represents a non-empty list, as evidenced by the fact that there is no [] variant. The sexp representation is as a regular list (i.e., the same as the Stable.V3 module below).

include Ppx_compare_lib.Comparable.S1 with type 'a t := 'a t
Sourceval compare : 'a Base__Ppx_compare_lib.compare -> 'a t Base__Ppx_compare_lib.compare
include Ppx_compare_lib.Equal.S1 with type 'a t := 'a t
Sourceval equal : 'a Base__Ppx_compare_lib.equal -> 'a t Base__Ppx_compare_lib.equal
include Sexplib0.Sexpable.S1 with type 'a t := 'a t
Sourceval t_of_sexp : (Sexplib0.Sexp.t -> 'a) -> Sexplib0.Sexp.t -> 'a t
Sourceval sexp_of_t : ('a -> Sexplib0.Sexp.t) -> 'a t -> Sexplib0.Sexp.t
include Ppx_hash_lib.Hashable.S1 with type 'a t := 'a t
Sourceval hash_fold_t : 'a Base__Ppx_hash_lib.hash_fold -> 'a t Base__Ppx_hash_lib.hash_fold
include Ppx_quickcheck_runtime.Quickcheckable.S1 with type 'a t := 'a t
include Core.Bin_prot.Binable.S1 with type 'a t := 'a t
Sourceval bin_size_t : ('a, 'a t) Bin_prot.Size.sizer1
Sourceval bin_write_t : ('a, 'a t) Bin_prot.Write.writer1
Sourceval bin_read_t : ('a, 'a t) Bin_prot.Read.reader1
Sourceval __bin_read_t__ : ('a, int -> 'a t) Bin_prot.Read.reader1
Sourceval bin_writer_t : ('a, 'a t) Bin_prot.Type_class.S1.writer
Sourceval bin_reader_t : ('a, 'a t) Bin_prot.Type_class.S1.reader
Sourceval bin_t : ('a, 'a t) Bin_prot.Type_class.S1.t
include Core.Comparator.Derived with type 'a t := 'a t
type !'cmp comparator_witness
val comparator : ('a, 'cmp) Base.Comparator.t -> ('a t, 'cmp comparator_witness) Base.Comparator.t
include Core.Container.S1 with type 'a t := 'a t
include Core.Invariant.S1 with type 'a t := 'a t
Sourceval invariant : 'a Base__Invariant_intf.inv -> 'a t Base__Invariant_intf.inv
include Core.Monad.S with type 'a t := 'a t
val (>>=) : 'a t -> ('a -> 'b t) -> 'b t
val (>>|) : 'a t -> ('a -> 'b) -> 'b t
module Monad_infix : sig ... end
val bind : 'a t -> f:('a -> 'b t) -> 'b t
val return : 'a -> 'a t
val map : 'a t -> f:('a -> 'b) -> 'b t
val join : 'a t t -> 'a t
val ignore_m : 'a t -> unit t
val all : 'a t list -> 'a list t
val all_unit : unit t list -> unit t
module Let_syntax : sig ... end
include Core.Indexed_container.S1 with type 'a t := 'a t
Sourceval mem : 'a t -> 'a -> equal:('a -> 'a -> bool) -> bool
Sourceval length : 'a t -> int
Sourceval is_empty : 'a t -> bool
Sourceval iter : 'a t -> f:('a -> unit) -> unit
Sourceval fold : 'a t -> init:'acc -> f:('acc -> 'a -> 'acc) -> 'acc
Sourceval fold_result : 'a t -> init:'acc -> f:('acc -> 'a -> ('acc, 'e) Base__.Result.t) -> ('acc, 'e) Base__.Result.t
Sourceval fold_until : 'a t -> init:'acc -> f:('acc -> 'a -> ('acc, 'final) Base__Container_intf.Continue_or_stop.t) -> finish:('acc -> 'final) -> 'final
Sourceval exists : 'a t -> f:('a -> bool) -> bool
Sourceval for_all : 'a t -> f:('a -> bool) -> bool
Sourceval count : 'a t -> f:('a -> bool) -> int
Sourceval sum : (module Base__Container_intf.Summable with type t = 'sum) -> 'a t -> f:('a -> 'sum) -> 'sum
Sourceval find : 'a t -> f:('a -> bool) -> 'a option
Sourceval find_map : 'a t -> f:('a -> 'b option) -> 'b option
Sourceval to_list : 'a t -> 'a list
Sourceval to_array : 'a t -> 'a array
Sourceval min_elt : 'a t -> compare:('a -> 'a -> int) -> 'a option
Sourceval max_elt : 'a t -> compare:('a -> 'a -> int) -> 'a option
Sourceval foldi : ('a t, 'a, 'b) Base__Indexed_container_intf.foldi
Sourceval existsi : 'a t -> f:(int -> 'a -> bool) -> bool
Sourceval for_alli : 'a t -> f:(int -> 'a -> bool) -> bool
Sourceval counti : 'a t -> f:(int -> 'a -> bool) -> int
Sourceval findi : 'a t -> f:(int -> 'a -> bool) -> (int * 'a) option
Sourceval find_mapi : 'a t -> f:(int -> 'a -> 'b option) -> 'b option
Sourceval create : 'a -> 'a list -> 'a t
Sourceval init : int -> f:(int -> 'a) -> 'a t
Sourceval of_list : 'a list -> 'a t option
Sourceval of_list_error : 'a list -> 'a t Core.Or_error.t
Sourceval of_list_exn : 'a list -> 'a t
Sourceval singleton : 'a -> 'a t
Sourceval cons : 'a -> 'a t -> 'a t
Sourceval hd : 'a t -> 'a
Sourceval tl : 'a t -> 'a list
Sourceval nth : 'a t -> int -> 'a option
Sourceval nth_exn : 'a t -> int -> 'a
Sourceval reduce : 'a t -> f:('a -> 'a -> 'a) -> 'a
Sourceval reverse : 'a t -> 'a t
Sourceval append : 'a t -> 'a list -> 'a t
Sourceval unzip : ('a * 'b) t -> 'a t * 'b t
Sourceval zip : 'a t -> 'b t -> ('a * 'b) t Core.List.Or_unequal_lengths.t
Sourceval zip_exn : 'a t -> 'b t -> ('a * 'b) t
Sourceval mapi : 'a t -> f:(int -> 'a -> 'b) -> 'b t
Sourceval filter : 'a t -> f:('a -> bool) -> 'a list
Sourceval filteri : 'a t -> f:(int -> 'a -> bool) -> 'a list
Sourceval filter_map : 'a t -> f:('a -> 'b option) -> 'b list
Sourceval filter_mapi : 'a t -> f:(int -> 'a -> 'b option) -> 'b list
Sourceval concat : 'a t t -> 'a t
Sourceval concat_map : 'a t -> f:('a -> 'b t) -> 'b t
Sourceval last : 'a t -> 'a
Sourceval drop_last : 'a t -> 'a list
Sourceval to_sequence : 'a t -> 'a Core.Sequence.t
Sourceval sort : 'a t -> compare:('a -> 'a -> int) -> 'a t
Sourceval stable_sort : 'a t -> compare:('a -> 'a -> int) -> 'a t
Sourceval dedup_and_sort : compare:('a -> 'a -> int) -> 'a t -> 'a t
Sourceval iteri : 'a t -> f:(int -> 'a -> unit) -> unit
Sourceval fold_right : 'a t -> init:'b -> f:('a -> 'b -> 'b) -> 'b
Sourceval folding_map : 'a t -> init:'b -> f:('b -> 'a -> 'b * 'c) -> 'c t
Sourceval fold_map : 'a t -> init:'acc -> f:('acc -> 'a -> 'acc * 'b) -> 'acc * 'b t
Sourceval min_elt' : 'a t -> compare:('a -> 'a -> int) -> 'a

min_elt' and max_elt' differ from min_elt and max_elt (included in Container.S1) in that they don't return options.

Sourceval max_elt' : 'a t -> compare:('a -> 'a -> int) -> 'a
Sourceval map_add_multi : ('k, 'v t, 'cmp) Core.Map.t -> key:'k -> data:'v -> ('k, 'v t, 'cmp) Core.Map.t

Like Map.add_multi, but comes with a guarantee that the range of the returned map is all nonempty lists.

Sourceval map_of_alist_multi : ('k * 'v) list -> comparator:('k, 'cmp) Core.Comparator.Module.t -> ('k, 'v t, 'cmp) Core.Map.t

Like Map.of_alist_multi, but comes with a guarantee that the range of the returned map is all nonempty lists.

Sourceval map_of_sequence_multi : ('k * 'v) Core.Sequence.t -> comparator:('k, 'cmp) Core.Comparator.Module.t -> ('k, 'v t, 'cmp) Core.Map.t

Like Map.of_sequence_multi, but comes with a guarantee that the range of the returned map is all nonempty lists.

Sourceval combine_errors : ('ok, 'err) Core.Result.t t -> ('ok t, 'err t) Core.Result.t

Like Result.combine_errors but for non-empty lists

Sourceval combine_errors_unit : (unit, 'err) Core.Result.t t -> (unit, 'err t) Core.Result.t

Like Result.combine_errors_unit but for non-empty lists

Sourceval combine_or_errors : 'a Core.Or_error.t t -> 'a t Core.Or_error.t

Like Or_error.combine_errors but for non-empty lists

Sourceval combine_or_errors_unit : unit Core.Or_error.t t -> unit Core.Or_error.t

Like Or_error.combine_errors_unit but for non-empty lists

Sourceval validate_indexed : 'a Core.Validate.check -> 'a t Core.Validate.check

validates a list, naming each element by its position in the list (where the first position is 1, not 0).

Sourceval validate : name:('a -> string) -> 'a Core.Validate.check -> 'a t Core.Validate.check

validates a list, naming each element using a user-defined function for computing the name.

Sourceval flag : 'a Core.Command.Param.Arg_type.t -> 'a t Core.Command.Flag.t

Returns a flag that must be passed one or more times. See Command.Param.one_or_more_as_pair.

Sourceval comma_separated_argtype : ?key:'a t Core.Univ_map.Multi.Key.t -> ?strip_whitespace:bool -> ?unique_values:bool -> 'a Core.Command.Param.Arg_type.t -> 'a t Core.Command.Param.Arg_type.t

Accepts comma-separated lists of arguments parsed by t. See Command.Param.Arg_type.comma_separated.

Sourcetype 'a nonempty_list := 'a t
Sourcemodule Option : sig ... end

This module provides 0-alloc versions of to_list and of_list, via some and allowing you to match%optional on a list, respectively.

Sourcemodule Reversed : sig ... end

a non-empty version of Reversed_list.t

Sourceval rev' : 'a t -> 'a Reversed.t
Sourceval rev_append : 'a Reversed_list.t -> 'a t -> 'a t
Sourcemodule Unstable : sig ... end
Sourcemodule Stable : sig ... end