package core_kernel

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Univ_map.With_fold

keys that map to an accumulator value with an associated fold operation

keys that map to an accumulator value with an associated fold operation

module Key : sig ... end
val set : Univ_map.t -> ('a, 'b) Key.t -> 'b -> Univ_map.t

reset the accumulator

val find : Univ_map.t -> ('a, 'b) Key.t -> 'b

the current accumulator

val add : Univ_map.t -> ('a, 'b) Key.t -> 'a -> Univ_map.t

fold value into accumulator

val change : Univ_map.t -> ('a, 'b) Key.t -> f:('b -> 'b) -> Univ_map.t

accumulator update